#include <math.h>
#include "descriptor/descriptor.h"

#define PIGR 3.14159265
//#define ANIMAZIONE

int main(void) {

	/* DEFINIZIONI */

	/* numero progressivo di frame per l'animazione */
	int frame;

	/* scena */
	ITEM scena, stuka;

	/* elementi della scena: pareti, tavolo, teca, lampadario */
	ITEM pareti, pavimento, tavolo, lampadario, lampadina, baseTeca, teca;

	/* elementi del modello */
	ITEM partiAzzurre, partiNere, elica, bomba, bombePiccole, mozzo, partiSabbia, ruote, calotta,
		fusoliera, ali, carrelli, alaPost;

	/* attributi materiali */
	ITEM intonaco, legno, marmo, vetro, azzurro65, grigio116, nero33, sabbia29;

	/* attributi luci */
	ITEM luceLampadina, luceAngolo, luceEsplosione, luceCorrezione1, luceCorrezione2;


	/* SETTING LUCI */

	/*
	AVVERTENZA PER IL SET DI LUCI
	In caso di resa senza ombre è possibile eliminare le luci di correzione e spostare
	luceLampadina alle coordinate 0.0,0.0,160.0
	*/

	set_background(0.8,0.8,0.8,FALSE);	/* lo sfondo non riflette la luce */
	set_ambient_light(1,1,1,1);

	/* luce della lampadina */
	luceLampadina = create_warn_light(0.0,0.0,149.0, 0.0,0.0,-1.0, 1,1,0.5, 5,200,0.01,45, 0,0,0,0,0, "luceLampadina");

	/* luce puntiforme */
	luceAngolo = create_point_light(-199.0,201.0,100.0, 1,1,0.9, 2, 450, "luceAngolo");

	/* luci per ovviare al problema dell'attraversamento delle superfici trasparenti */
	luceCorrezione1 = create_point_light(25.0,-30.0,10.0, 1,1,0.5, 2, 30, "luceCorrezione1");
	luceCorrezione2 = create_point_light(0.0,0.0,155.0, 1,1,0.5, 5, 20, "luceCorrezione2");


	/* SETTING ATTRIBUTI */

	/* 
	GUIDA AI PARAMETRI DELLE FUNZIONI
	set_color(item, R, G, B, luceAmbienteRiflessa, luceDiffusa)
	set_reflectivity(item, luceSpeculareRiflessa, esponente per Phong)
	set_transparency(item, maxt, mint, tpwr, indiceRifrazione)
		t = (maxt − mint) ∗ (1 − (1 − cos(alpha))^tpwr ) + mint
	*/

	/* intonaco */
	intonaco = create_attribute("intonaco");
	set_color(intonaco,0.90,0.90,0.90,0.5,0.8);
	set_reflectivity(intonaco,0.0,0);

	/* legno */
	legno = create_attribute("legno");
	set_color(legno,0.6,0.4,0.3,0.1,0.8);
	set_reflectivity(legno,0.1,0);

	/* marmo */
	marmo=create_attribute("marmo");
	set_color(marmo,1,0.9,0.9,0.5,0.5);
	set_reflectivity(marmo,0.2,1);

	/* vetro */
	vetro = create_attribute("vetro");
	set_color(vetro,0.9,0.9,1,0.6,0.6);
	set_reflectivity(vetro,0.35,3);
	set_transparency(vetro,1,0.99,2,1.01);

	/* vernici modellino: azzurro65, grigio116, nero33, sabbia29 */
	azzurro65 = create_attribute("azzurro65");
	set_color(azzurro65,0.40,0.85,0.93,0.2,0.45);
	set_reflectivity(azzurro65,0.05,1);

	grigio116 = create_attribute("grigio116");
	set_color(grigio116,0.3,0.3,0.3,0.2,0.45);
	set_reflectivity(grigio116,0.05,1);

	nero33 = create_attribute("nero33");
	set_color(nero33,0.0,0.0,0.0,0.2,0.45);
	set_reflectivity(nero33,0.05,1);

	sabbia29 = create_attribute("sabbia29");
	set_color(sabbia29,0.7,0.5,0.3,0.2,0.45);
	set_reflectivity(sabbia29,0.05,1);


	/* LOAD DELLA SCENA */

	/* pavimento */
	pavimento = read_nurbs("pavimento.db");
	set_domain_texture(pavimento,legno,"textures/parquet.hr",0,0.175,0,0.175,TRUE,0.1,0.8);
	scena = create_list(pavimento,"scena");

	/* 3 pareti laterali e soffitto */
	pareti = read_obj("pareti.obj","pareti");
	set_domain_texture(pareti,intonaco,"textures/intonaco.hr",0,0.2,0,0.2,TRUE,0.5,0.8);
	scena = insert_in_list(scena,pareti);

	/* tavolo */
	tavolo = read_nurbs("tavolo.db");
	set_scale(tavolo, 0.01, 0.01, 0.01);
	set_projection_texture(tavolo,marmo,"textures/marmo.hr",2,TRUE,0.5,0.5);
	set_scale(tavolo,100,100,100);
	scena = insert_in_list(scena,tavolo);

	/* teca */
	baseTeca = read_nurbs("baseTeca.db");
	set_scale(baseTeca,0.125,0.125,0.125);
	set_projection_texture(baseTeca,legno,"textures/legno.hr",2,TRUE,0.1,0.8);
	set_scale(baseTeca,8,8,8);
	scena = insert_in_list(scena,baseTeca);
	teca = read_nurbs("teca.dbe");
	set_attribute(teca,vetro);
	set_x_rotate(teca,3.2,1);	/* correzione posizione teca di vetro */
	set_z_rotate(teca,45,1);
	set_translate(teca,15.0,-30.0,-0.6);
	scena = insert_in_list(scena,teca);

	/* lampadario */
	lampadina = read_nurbs("lampadina.db");
	set_attribute(lampadina,vetro);
	scena = insert_in_list(scena,lampadina);
	lampadario = read_nurbs("lampadario.db");
	set_domain_texture(lampadario,marmo,"textures/lampadario.hr",0,1,0,1,TRUE,0.5,0.5);
	scena = insert_in_list(scena,lampadario);

	
	/* LOAD DEL MODELLO */

	/* parti non texturate */
	partiAzzurre = read_obj("ColAzzurro.obj","partiAzzurre");
	set_attribute(partiAzzurre,azzurro65);
	stuka = create_list(partiAzzurre,"stuka");

	partiNere = read_obj("ColNero.obj","partiNere");
	set_attribute(partiNere,nero33);
	stuka = insert_in_list(stuka,partiNere);

	elica = read_obj("elica.obj","elica");
	set_attribute(elica,grigio116);
	stuka = insert_in_list(stuka,elica);

	bomba = read_obj("bomba.obj","bomba");
	set_attribute(bomba,grigio116);
	stuka = insert_in_list(stuka,bomba);

	bombePiccole = read_obj("bombePiccole.obj","bombePiccole");
	set_attribute(bombePiccole,grigio116);
	stuka = insert_in_list(stuka,bombePiccole);

	partiSabbia = read_obj("ColSabbia.obj","partiSabbia");
	set_attribute(partiSabbia,sabbia29);
	stuka = insert_in_list(stuka,partiSabbia);

	/* parti con la texture mimetica per proiezione */
	fusoliera = read_nurbs("fusoliera1.db");
	set_scale(fusoliera,0.5,0.5,0.5);
	set_projection_texture(fusoliera,sabbia29,"textures/mimetico.hr",1,TRUE,0.05,1);
	set_scale(fusoliera,2,2,2);
	stuka = insert_in_list(stuka,fusoliera);

	ali = read_obj("ali1.obj","ali");
	set_scale(ali,0.5,0.5,0.5);
	set_projection_texture(ali,sabbia29,"textures/mimetico.hr",2,TRUE,0.05,1);
	set_scale(ali,2,2,2);
	stuka = insert_in_list(stuka,ali);

	carrelli = read_obj("carrelli.obj","carrelli");
	set_scale(carrelli,0.5,0.5,0.5);
	set_projection_texture(carrelli,sabbia29,"textures/mimetico.hr",1,TRUE,0.05,1);
	set_scale(carrelli,2,2,2);
	stuka = insert_in_list(stuka,carrelli);

	alaPost = read_nurbs("alaPost1.db");
	set_scale(alaPost,0.5,0.5,0.5);
	set_projection_texture(alaPost,sabbia29,"textures/mimetico.hr",2,TRUE,0.05,1);
	set_scale(alaPost,2,2,2);
	stuka = insert_in_list(stuka,alaPost);

	/* calotta del pilota */
	calotta = read_nurbs("calotta.db");
	set_domain_texture(calotta,azzurro65,"textures/calotta.hr",0,1,0,1,TRUE,0.05,1);
	stuka = insert_in_list(stuka,calotta);
	
	/* parti ottenute per rotazione (domain texture) */
	mozzo = read_nurbs("mozzo.db");
	set_domain_texture(mozzo,grigio116,"textures/mozzo.hr",0,1,0,1,TRUE,0.05,1);
	stuka = insert_in_list(stuka,mozzo);

	ruote = read_obj("ruote.obj","ruote");
	set_domain_texture(ruote,grigio116,"textures/ruota.hr",0,1,0,1,TRUE,0.05,1);
	stuka = insert_in_list(stuka,ruote);

	/* trasformazioni applicate a tutto il modello quando è fermo */
	set_translate(stuka,0.0,0.0,0.8);
	set_y_rotate(stuka,12.3,1);

	scena = insert_in_list(scena,stuka);


	/* RESA */

#ifndef ANIMAZIONE

	/* resa singola */
	save_scene("stuka.md","stuka",scena);

#else
	
	/* animazione .ani */
	for (frame = 1; frame <= 100; frame++) {

		/* set neutro */
		set_translate_abs(stuka,0.0,0.0,0.0);

		/* ROTAZIONI */
	
		/* rotazione elica con accelerazione iniziale */
		set_translate(stuka,0.0,0.0,-0.7);
		if (frame <= 10) set_x_rotate(elica,5*frame*frame,1);
		else set_x_rotate(elica,50*frame,1);
		set_translate(stuka,0.0,0.0,0.7);

		/* carrello posteriore appoggiato a terra */
		set_translate(stuka,0.0,0.0,0.8);
		if (frame <= 25) set_y_rotate(stuka,12.3,1);

		/* la coda si alza */
		if ((frame > 21) && (frame <= 25)) set_y_rotate(stuka,-3*(frame-21),1);

		/* beccheggio */
		if ((frame >= 26) && (frame < 30)) set_y_rotate(stuka,frame-23,1);
		if (frame >= 30) set_y_rotate(stuka,7,1);

		/* rollio e imbardata */
		if ((frame >= 41) && (frame <= 70)) {
			if (frame < 55) set_x_rotate(stuka,-3*(frame-40),1);
			else set_x_rotate(stuka,-90+3*(frame-40),1);
			set_z_rotate(stuka,-3*(frame-40),1);
		}

		/* bomba */
		if (frame >= 76) set_y_rotate(bomba,-14,1);

		/* dopo la virata */
		if (frame > 70) set_z_rotate(stuka,-90,1);


		/* TRASLAZIONI */

		/* movimento  sul tavolo fino a x = -50 */	
		if ((frame > 10) && (frame <= 25))
			set_translate(stuka,-3.3333*(frame-10),0.0,0.0);

		/* movimento  in aria fino a x = -100 */	
		if ((frame > 25) && (frame <= 40))
			set_translate(stuka,-3.3333*(frame-25)-50,0.0,0.0);

		/* posizione durante la virata */
		if ((frame >= 41) && (frame <= 70))
			set_translate(stuka,-100 - 50 * sin(3*(frame-40)*PIGR/180),
					    50 - 50 * cos(3*(frame-40)*PIGR/180),0.0);

		/* posizione dopo la virata */
		if (frame >= 71) set_translate(stuka,-150,4.17*(frame-70)+50,0.0);

		/* spostamento in verticale */
		if (frame == 26) set_translate(stuka,0.0,0.0,0.187);	/* 3 gradi */
		if (frame == 27) set_translate(stuka,0.0,0.0,0.437);	/* 4 gradi */
		if (frame == 28) set_translate(stuka,0.0,0.0,0.749);	/* 5 gradi */
		if (frame == 29) set_translate(stuka,0.0,0.0,1.124);	/* 6 gradi */
		if ((frame >= 30) && (frame <= 40)) set_translate(stuka,0.0,0.0,0.41*(frame-29)+1.124);
		if ((frame >= 41) && (frame <= 70)) set_translate(stuka,0.0,0.0,0.322*(frame-40)+5.224);
		if (frame >= 71) set_translate(stuka,0.0,0.0,0.512*(frame-70)+14.884);

		/* caduta della bomba e esplosione*/
		if (frame >= 76) set_translate(bomba,0.0,0.0,-1.024*(frame-75));
		if (frame == 100) {
			set_translate(bomba,0.0,100.0,0.0); /* la bomba viene spostata fuori dalla scena */
			luceEsplosione = create_point_light(-150.0,175.1,4.644, 1,1,1, 100, 800, "luceEsplosione");
		}


		save_scene_ani(100,"stuka.ani","stuka","stuka",scena);
	}

#endif

	return(0);

}






